@@ -59,7 +59,7 @@ struct _PpNewPrinterPrivate
   guint     window_id;
   gboolean  unlink_ppd_file;
 
-  GSimpleAsyncResult *res;
+  GTask        *task;
   GCancellable *cancellable;
 };
 
@@ -102,12 +102,8 @@ pp_new_printer_finalize (GObject *object)
   g_clear_pointer (&priv->location, g_free);
   g_clear_pointer (&priv->make_and_model, g_free);
   g_clear_pointer (&priv->host_name, g_free);
-
-  if (priv->res)
-    g_object_unref (priv->res);
-
-  if (priv->cancellable)
-    g_object_unref (priv->cancellable);
+  g_clear_object (&priv->task);
+  g_clear_object (&priv->cancellable);
 
   G_OBJECT_CLASS (pp_new_printer_parent_class)->finalize (object);
 }
@@ -350,8 +346,6 @@ pp_new_printer_init (PpNewPrinter *printer)
                                                PpNewPrinterPrivate);
 
   printer->priv->unlink_ppd_file = FALSE;
-  printer->priv->cancellable = NULL;
-  printer->priv->res = NULL;
 }
 
 PpNewPrinter *
@@ -370,14 +364,14 @@ _pp_new_printer_add_async_cb (gboolean      success,
 
   if (!success)
     {
-      g_simple_async_result_set_error (priv->res,
-                                       G_IO_ERROR,
-                                       G_IO_ERROR_FAILED,
-                                       "Installation of the new printer failed.");
+      g_task_return_new_error (priv->task,
+                               G_IO_ERROR,
+                               G_IO_ERROR_FAILED,
+                               "Installation of the new printer failed.");
+      return;
     }
 
-  g_simple_async_result_set_op_res_gboolean (priv->res, success);
-  g_simple_async_result_complete_in_idle (priv->res);
+  g_task_return_boolean (priv->task, success);
 }
 
 static void
@@ -432,14 +426,11 @@ printer_add_real_async_dbus_cb (GObject      *source_object,
     }
   else
     {
-      if (error->domain != G_IO_ERROR ||
-          error->code != G_IO_ERROR_CANCELLED)
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         g_warning ("%s", error->message);
     }
 
-  if (!error ||
-      error->domain != G_IO_ERROR ||
-      error->code != G_IO_ERROR_CANCELLED)
+  if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
     {
       get_named_dest_async (priv->name,
                             printer_add_real_async_cb,
@@ -570,14 +561,11 @@ printer_add_async_scb3 (GObject      *source_object,
     }
   else
     {
-      if (error->domain != G_IO_ERROR ||
-          error->code != G_IO_ERROR_CANCELLED)
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         g_warning ("%s", error->message);
     }
 
-  if ((!error ||
-      error->domain != G_IO_ERROR ||
-      error->code != G_IO_ERROR_CANCELLED) &&
+  if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
       ppd_item && ppd_item->ppd_name)
     {
       priv->ppd_name = g_strdup (ppd_item->ppd_name);
@@ -616,14 +604,11 @@ install_printer_drivers_cb (GObject      *source_object,
     }
   else
     {
-      if (error->domain != G_IO_ERROR ||
-          error->code != G_IO_ERROR_CANCELLED)
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         g_warning ("%s", error->message);
     }
 
-  if (!error ||
-      error->domain != G_IO_ERROR ||
-      error->code != G_IO_ERROR_CANCELLED)
+  if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
     {
       GDBusConnection  *bus;
       g_autoptr(GError) bus_error = NULL;
@@ -808,8 +793,7 @@ printer_configure_async_finish (PCData *data)
     {
       _pp_new_printer_add_async_cb (TRUE, data->new_printer);
 
-      if (data->cancellable)
-        g_object_unref (data->cancellable);
+      g_clear_object (&data->cancellable);
       g_free (data);
     }
 }
@@ -844,8 +828,7 @@ printer_set_accepting_jobs_cb (GObject      *source_object,
     }
   else
     {
-      if (error->domain != G_IO_ERROR ||
-          error->code != G_IO_ERROR_CANCELLED)
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         g_warning ("%s", error->message);
     }
 
@@ -873,8 +856,7 @@ printer_set_enabled_cb (GObject      *source_object,
     }
   else
     {
-      if (error->domain != G_IO_ERROR ||
-          error->code != G_IO_ERROR_CANCELLED)
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         g_warning ("%s", error->message);
     }
 
@@ -944,8 +926,7 @@ install_package_names_cb (GObject      *source_object,
     }
   else
     {
-      if (error->domain != G_IO_ERROR ||
-          error->code != G_IO_ERROR_CANCELLED)
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         g_warning ("%s", error->message);
     }
 
@@ -982,8 +963,7 @@ search_files_cb (GObject      *source_object,
     }
   else
     {
-      if (error->domain != G_IO_ERROR ||
-          error->code != G_IO_ERROR_CANCELLED)
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         g_warning ("%s", error->message);
     }
 
@@ -1107,8 +1087,7 @@ DBus method \"MissingExecutables\" to find missing executables and filters.");
     }
   else
     {
-      if (error->domain != G_IO_ERROR ||
-          error->code != G_IO_ERROR_CANCELLED)
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         g_warning ("%s", error->message);
     }
 
@@ -1204,8 +1183,7 @@ pp_maintenance_command_execute_cb (GObject      *source_object,
     }
   else
     {
-      if (error->domain != G_IO_ERROR ||
-          error->code != G_IO_ERROR_CANCELLED)
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         {
           data = (PCData *) user_data;
 
@@ -1323,15 +1301,16 @@ printer_configure_async (PpNewPrinter *new_printer)
                          ime_data);
 }
 
-static void
-_pp_new_printer_add_async (GSimpleAsyncResult *res,
-                           GObject            *object,
-                           GCancellable       *cancellable)
+void
+pp_new_printer_add_async (PpNewPrinter        *printer,
+                          GCancellable        *cancellable,
+                          GAsyncReadyCallback  callback,
+                          gpointer             user_data)
 {
-  PpNewPrinter        *printer = PP_NEW_PRINTER (object);
   PpNewPrinterPrivate *priv = printer->priv;
+  g_autoptr(GTask) task = NULL;
 
-  priv->res = g_object_ref (res);
+  priv->task = g_task_new (printer, cancellable, callback, user_data);
   priv->cancellable = g_object_ref (cancellable);
 
   if (priv->ppd_name || priv->ppd_file_name)
@@ -1385,33 +1364,12 @@ _pp_new_printer_add_async (GSimpleAsyncResult *res,
     }
 }
 
-void
-pp_new_printer_add_async (PpNewPrinter        *printer,
-                          GCancellable        *cancellable,
-                          GAsyncReadyCallback  callback,
-                          gpointer             user_data)
-{
-  GSimpleAsyncResult *res;
-
-  res = g_simple_async_result_new (G_OBJECT (printer), callback, user_data, pp_new_printer_add_async);
-
-  g_simple_async_result_set_check_cancellable (res, cancellable);
-  _pp_new_printer_add_async (res, G_OBJECT (printer), cancellable);
-
-  g_object_unref (res);
-}
-
 gboolean
 pp_new_printer_add_finish (PpNewPrinter  *printer,
                            GAsyncResult  *res,
                            GError       **error)
 {
-  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (res);
-
-  g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == pp_new_printer_add_async);
-
-  if (g_simple_async_result_propagate_error (simple, error))
-    return FALSE;
-
-  return g_simple_async_result_get_op_res_gboolean (simple);
+  g_return_val_if_fail (g_task_is_valid (res, printer), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+  return g_task_propagate_boolean (G_TASK (res), error);
 }
