@@ -35,8 +35,8 @@
 #define REPORT_TECHNICAL_PROBLEMS "report-technical-problems"
 #define LOCATION_ENABLED "enabled"
 
-#define APP_PERMISSIONS_TABLE "gnome"
-#define APP_PERMISSIONS_ID "geolocation"
+#define APP_PERMISSIONS_TABLE "location"
+#define APP_PERMISSIONS_ID "location"
 
 struct _CcPrivacyPanel
 {
@@ -56,6 +56,12 @@ struct _CcPrivacyPanel
   GtkDialog   *location_dialog;
   GtkLabel    *location_label;
   GtkSwitch   *location_services_switch;
+  GtkDialog   *camera_dialog;
+  GtkLabel    *camera_label;
+  GtkSwitch   *camera_switch;
+  GtkDialog   *microphone_dialog;
+  GtkLabel    *microphone_label;
+  GtkSwitch   *microphone_switch;
   GtkComboBox *lock_after_combo;
   GtkLabel    *lock_after_label;
   GtkComboBox *purge_after_combo;
@@ -303,8 +309,12 @@ add_row (CcPrivacyPanel *self,
 
   row = gtk_list_box_row_new ();
   gtk_widget_show (row);
-  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 50);
+  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 20);
   gtk_widget_show (box);
+  gtk_widget_set_margin_start (box, 20);
+  gtk_widget_set_margin_end (box, 20);
+  gtk_widget_set_margin_top (box, 18);
+  gtk_widget_set_margin_bottom (box, 18);
   gtk_container_add (GTK_CONTAINER (row), box);
   g_object_set_data (G_OBJECT (row), "dialog", dialog);
   gtk_widget_set_hexpand (box, TRUE);
@@ -313,16 +323,12 @@ add_row (CcPrivacyPanel *self,
   w = gtk_label_new (label);
   gtk_widget_show (w);
   gtk_widget_set_halign (w, GTK_ALIGN_START);
-  gtk_widget_set_margin_start (w, 20);
-  gtk_widget_set_margin_end (w, 20);
-  gtk_widget_set_margin_top (w, 18);
-  gtk_widget_set_margin_bottom (w, 18);
   gtk_widget_set_halign (w, GTK_ALIGN_START);
   gtk_widget_set_valign (w, GTK_ALIGN_CENTER);
   gtk_widget_set_hexpand (w, TRUE);
+  gtk_label_set_ellipsize (GTK_LABEL (w), PANGO_ELLIPSIZE_END);
+  gtk_label_set_xalign (GTK_LABEL (w), 0.0f);
   gtk_box_pack_start (GTK_BOX (box), w, TRUE, TRUE, 0);
-  gtk_widget_set_margin_start (status, 20);
-  gtk_widget_set_margin_end (status, 20);
   gtk_widget_set_halign (status, GTK_ALIGN_END);
   gtk_widget_set_valign (status, GTK_ALIGN_CENTER);
   gtk_box_pack_end (GTK_BOX (box), status, FALSE, FALSE, 0);
@@ -868,6 +874,72 @@ add_location (CcPrivacyPanel *self)
                             self);
 }
 
+static void
+update_camera_label (CcPrivacyPanel *self)
+{
+  if (g_settings_get_boolean (self->privacy_settings, "disable-camera"))
+    gtk_label_set_label (self->camera_label, C_("Camera status", "Off"));
+  else
+    gtk_label_set_label (self->camera_label, C_("Camera status", "On"));
+}
+
+static void
+add_camera (CcPrivacyPanel *self)
+{
+  self->camera_label = GTK_LABEL (gtk_label_new (""));
+  gtk_widget_show (GTK_WIDGET (self->camera_label));
+  update_camera_label (self);
+
+  add_row (self,
+           _("Camera"),
+           self->camera_dialog,
+           GTK_WIDGET (self->camera_label));
+
+  g_signal_connect (self->camera_dialog, "delete-event",
+                    G_CALLBACK (gtk_widget_hide_on_delete), NULL);
+
+  g_settings_bind (self->privacy_settings, "disable-camera",
+                   self->camera_switch, "active",
+                   G_SETTINGS_BIND_INVERT_BOOLEAN);
+
+  g_signal_connect_object (self->privacy_settings, "changed::disable-camera",
+                           G_CALLBACK (update_camera_label), self,
+                           G_CONNECT_SWAPPED);
+}
+
+static void
+update_microphone_label (CcPrivacyPanel *self)
+{
+  if (g_settings_get_boolean (self->privacy_settings, "disable-microphone"))
+    gtk_label_set_label (self->microphone_label, C_("Microphone status", "Off"));
+  else
+    gtk_label_set_label (self->microphone_label, C_("Microphone status", "On"));
+}
+
+static void
+add_microphone (CcPrivacyPanel *self)
+{
+  self->microphone_label = GTK_LABEL (gtk_label_new (""));
+  gtk_widget_show (GTK_WIDGET (self->microphone_label));
+  update_microphone_label (self);
+
+  add_row (self,
+           _("Microphone"),
+           self->microphone_dialog,
+           GTK_WIDGET (self->microphone_label));
+
+  g_signal_connect (self->microphone_dialog, "delete-event",
+                    G_CALLBACK (gtk_widget_hide_on_delete), NULL);
+
+  g_settings_bind (self->privacy_settings, "disable-microphone",
+                   self->microphone_switch, "active",
+                   G_SETTINGS_BIND_INVERT_BOOLEAN);
+
+  g_signal_connect_object (self->privacy_settings, "changed::disable-microphone",
+                           G_CALLBACK (update_microphone_label), self,
+                           G_CONNECT_SWAPPED);
+}
+
 static void
 retain_history_combo_changed_cb (GtkWidget      *widget,
                                  CcPrivacyPanel *self)
@@ -1309,6 +1381,8 @@ cc_privacy_panel_init (CcPrivacyPanel *self)
 
   add_screen_lock (self);
   add_location (self);
+  add_camera (self);
+  add_microphone (self);
   add_usage_history (self);
   add_trash_temp (self);
   add_software (self);
@@ -1345,6 +1419,10 @@ cc_privacy_panel_class_init (CcPrivacyPanelClass *klass)
   gtk_widget_class_bind_template_child (widget_class, CcPrivacyPanel, location_apps_list_box);
   gtk_widget_class_bind_template_child (widget_class, CcPrivacyPanel, location_dialog);
   gtk_widget_class_bind_template_child (widget_class, CcPrivacyPanel, location_services_switch);
+  gtk_widget_class_bind_template_child (widget_class, CcPrivacyPanel, camera_dialog);
+  gtk_widget_class_bind_template_child (widget_class, CcPrivacyPanel, camera_switch);
+  gtk_widget_class_bind_template_child (widget_class, CcPrivacyPanel, microphone_dialog);
+  gtk_widget_class_bind_template_child (widget_class, CcPrivacyPanel, microphone_switch);
   gtk_widget_class_bind_template_child (widget_class, CcPrivacyPanel, lock_after_combo);
   gtk_widget_class_bind_template_child (widget_class, CcPrivacyPanel, lock_after_label);
   gtk_widget_class_bind_template_child (widget_class, CcPrivacyPanel, purge_after_combo);
