@@ -191,7 +191,7 @@ sort_regions (gconstpointer a,
 static GtkWidget *
 padded_label_new (char *text, gboolean narrow)
 {
-        GtkWidget *widget;
+        GtkWidget *widget, *label;
 
         widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
         gtk_widget_set_halign (widget, GTK_ALIGN_CENTER);
@@ -199,7 +199,9 @@ padded_label_new (char *text, gboolean narrow)
         gtk_widget_set_margin_bottom (widget, 10);
         gtk_widget_set_margin_start (widget, narrow ? 10 : 80);
         gtk_widget_set_margin_end (widget, narrow ? 10 : 80);
-        gtk_box_pack_start (GTK_BOX (widget), gtk_label_new (text), FALSE, FALSE, 0);
+        label = gtk_label_new (text);
+        gtk_widget_show (label);
+        gtk_container_add (GTK_CONTAINER (widget), label);
 
         return widget;
 }
@@ -222,22 +224,26 @@ region_widget_new (const gchar *locale_id,
         locale_untranslated_name = gnome_get_country_from_locale (locale_id, "C");
 
         row = gtk_list_box_row_new ();
+        gtk_widget_show (row);
         box = padded_label_new (locale_name, is_extra);
+        gtk_widget_show (box);
         gtk_container_add (GTK_CONTAINER (row), box);
 
         /* We add a check on each side of the label to keep it centered. */
         check = gtk_image_new ();
+        gtk_widget_show (check);
         gtk_image_set_from_icon_name (GTK_IMAGE (check), "object-select-symbolic", GTK_ICON_SIZE_MENU);
         gtk_widget_set_opacity (check, 0.0);
         g_object_set (check, "icon-size", GTK_ICON_SIZE_MENU, NULL);
-        gtk_box_pack_start (GTK_BOX (box), check, FALSE, FALSE, 0);
+        gtk_container_add (GTK_CONTAINER (box), check);
         gtk_box_reorder_child (GTK_BOX (box), check, 0);
 
         check = gtk_image_new ();
+        gtk_widget_show (check);
         gtk_image_set_from_icon_name (GTK_IMAGE (check), "object-select-symbolic", GTK_ICON_SIZE_MENU);
         gtk_widget_set_opacity (check, 0.0);
         g_object_set (check, "icon-size", GTK_ICON_SIZE_MENU, NULL);
-        gtk_box_pack_start (GTK_BOX (box), check, FALSE, FALSE, 0);
+        gtk_container_add (GTK_CONTAINER (box), check);
 
         g_object_set_data (G_OBJECT (row), "check", check);
         g_object_set_data_full (G_OBJECT (row), "locale-id", g_strdup (locale_id), g_free);
@@ -257,14 +263,17 @@ more_widget_new (void)
 
         row = gtk_list_box_row_new ();
         box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
+        gtk_widget_show (box);
         gtk_container_add (GTK_CONTAINER (row), box);
         gtk_widget_set_tooltip_text (box, _("More…"));
 
         arrow = gtk_image_new_from_icon_name ("view-more-symbolic", GTK_ICON_SIZE_MENU);
+        gtk_widget_show (arrow);
         gtk_style_context_add_class (gtk_widget_get_style_context (arrow), "dim-label");
+        gtk_widget_set_hexpand (arrow, TRUE);
         gtk_widget_set_margin_top (box, 10);
         gtk_widget_set_margin_bottom (box, 10);
-        gtk_box_pack_start (GTK_BOX (box), arrow, TRUE, TRUE, 0);
+        gtk_container_add (GTK_CONTAINER (box), arrow);
 
         return GTK_LIST_BOX_ROW (row);
 }
@@ -302,13 +311,12 @@ add_regions (CcFormatChooser *chooser,
                 if (!widget)
                   continue;
 
+                gtk_widget_show (widget);
                 gtk_container_add (GTK_CONTAINER (chooser->region_listbox), widget);
         }
 
         gtk_container_add (GTK_CONTAINER (chooser->region_listbox), GTK_WIDGET (chooser->more_item));
 
-        gtk_widget_show_all (chooser->region_listbox);
-
         chooser->adding = FALSE;
 }
 
@@ -409,9 +417,8 @@ show_more (CcFormatChooser *chooser)
 }
 
 static void
-row_activated (GtkListBox      *box,
-               GtkListBoxRow   *row,
-               CcFormatChooser *chooser)
+row_activated (CcFormatChooser *chooser,
+               GtkListBoxRow   *row)
 {
         const gchar *new_locale_id;
 
@@ -436,13 +443,12 @@ row_activated (GtkListBox      *box,
 }
 
 static void
-activate_default (GtkWindow       *window,
-                  CcFormatChooser *chooser)
+activate_default (CcFormatChooser *chooser)
 {
         GtkWidget *focus;
         const gchar *locale_id;
 
-        focus = gtk_window_get_focus (window);
+        focus = gtk_window_get_focus (GTK_WINDOW (chooser));
         if (!focus)
                 return;
 
@@ -450,7 +456,7 @@ activate_default (GtkWindow       *window,
         if (g_strcmp0 (locale_id, chooser->region) == 0)
                 return;
 
-        g_signal_stop_emission_by_name (window, "activate-default");
+        g_signal_stop_emission_by_name (chooser, "activate-default");
         gtk_widget_activate (focus);
 }
 
@@ -493,9 +499,10 @@ cc_format_chooser_init (CcFormatChooser *chooser)
         gtk_widget_init_template (GTK_WIDGET (chooser));
 
         chooser->more_item = more_widget_new ();
+        gtk_widget_show (GTK_WIDGET (chooser->more_item));
         /* We ref-sink here so we can reuse this widget multiple times */
         chooser->no_results = g_object_ref_sink (no_results_widget_new ());
-        gtk_widget_show_all (chooser->no_results);
+        gtk_widget_show (chooser->no_results);
 
         gtk_list_box_set_sort_func (GTK_LIST_BOX (chooser->region_listbox),
                                     (GtkListBoxSortFunc)sort_regions, chooser, NULL);
@@ -508,16 +515,16 @@ cc_format_chooser_init (CcFormatChooser *chooser)
 
         add_all_regions (chooser);
 
-        g_signal_connect_swapped (chooser->region_filter_entry, "search-changed",
-                                  G_CALLBACK (filter_changed), chooser);
+        g_signal_connect_object (chooser->region_filter_entry, "search-changed",
+                                 G_CALLBACK (filter_changed), chooser, G_CONNECT_SWAPPED);
 
-        g_signal_connect (chooser->region_listbox, "row-activated",
-                          G_CALLBACK (row_activated), chooser);
+        g_signal_connect_object (chooser->region_listbox, "row-activated",
+                                 G_CALLBACK (row_activated), chooser, G_CONNECT_SWAPPED);
 
         gtk_list_box_invalidate_filter (GTK_LIST_BOX (chooser->region_listbox));
 
-        g_signal_connect (chooser, "activate-default",
-                          G_CALLBACK (activate_default), chooser);
+        g_signal_connect_object (chooser, "activate-default",
+                                 G_CALLBACK (activate_default), chooser, G_CONNECT_SWAPPED);
 }
 
 CcFormatChooser *
