@@ -38,23 +38,21 @@
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnome-desktop/gnome-languages.h>
 
-#include "um-user-image.h"
-
 #include "cc-add-user-dialog.h"
+#include "cc-avatar-chooser.h"
+#include "cc-carousel.h"
 #include "cc-language-chooser.h"
 #include "cc-login-history-dialog.h"
 #include "cc-password-dialog.h"
+#include "cc-realm-manager.h"
 #include "cc-user-accounts-resources.h"
-#include "um-carousel.h"
-#include "um-photo-dialog.h"
+#include "cc-user-image.h"
 #include "um-fingerprint-dialog.h"
-#include "um-utils.h"
+#include "user-utils.h"
 
 #include "cc-common-language.h"
 #include "cc-util.h"
 
-#include "um-realm-manager.h"
-
 #define USER_ACCOUNTS_PERMISSION "org.gnome.controlcenter.user-accounts.administration"
 
 struct _CcUserPanel {
@@ -73,7 +71,7 @@ struct _CcUserPanel {
         GtkBox          *autologin_box;
         GtkLabel        *autologin_label;
         GtkSwitch       *autologin_switch;
-        UmCarousel      *carousel;
+        CcCarousel      *carousel;
         GtkButton       *fingerprint_button;
         GtkLabel        *fingerprint_label;
         GtkEntry        *full_name_entry;
@@ -91,15 +89,15 @@ struct _CcUserPanel {
         GtkButton       *remove_user_button;
         GtkStack        *stack;
         GtkToggleButton *user_icon_button;
-        UmUserImage     *user_icon_image;
-        UmUserImage     *user_icon_image2;
+        CcUserImage     *user_icon_image;
+        CcUserImage     *user_icon_image2;
         GtkStack        *user_icon_stack;
 
         ActUser *selected_user;
         GPermission *permission;
         CcLanguageChooser *language_chooser;
 
-        UmPhotoDialog *photo_dialog;
+        CcAvatarChooser *avatar_chooser;
 
         gint other_accounts;
 };
@@ -176,7 +174,7 @@ get_real_or_user_name (ActUser *user)
 static void show_user (ActUser *user, CcUserPanel *self);
 
 static void
-set_selected_user (CcUserPanel *self, UmCarouselItem *item)
+set_selected_user (CcUserPanel *self, CcCarouselItem *item)
 {
         uid_t uid;
 
@@ -198,8 +196,8 @@ create_carousel_entry (CcUserPanel *self, ActUser *user)
 
         box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
-        widget = um_user_image_new ();
-        um_user_image_set_user (UM_USER_IMAGE (widget), user);
+        widget = cc_user_image_new ();
+        cc_user_image_set_user (CC_USER_IMAGE (widget), user);
         gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 0);
 
         label = g_strdup_printf ("<b>%s</b>",
@@ -239,7 +237,7 @@ user_added (CcUserPanel *self, ActUser *user)
         g_debug ("user added: %d %s\n", act_user_get_uid (user), get_real_or_user_name (user));
 
         widget = create_carousel_entry (self, user);
-        item = um_carousel_item_new ();
+        item = cc_carousel_item_new ();
         gtk_container_add (GTK_CONTAINER (item), widget);
 
         g_object_set_data (G_OBJECT (item), "uid", GINT_TO_POINTER (act_user_get_uid (user)));
@@ -291,7 +289,7 @@ reload_users (CcUserPanel *self, ActUser *selected_user)
 {
         ActUser *user;
         GSList *list, *l;
-        UmCarouselItem *item = NULL;
+        CcCarouselItem *item = NULL;
         GtkSettings *settings;
         gboolean animations;
 
@@ -300,7 +298,7 @@ reload_users (CcUserPanel *self, ActUser *selected_user)
         g_object_get (settings, "gtk-enable-animations", &animations, NULL);
         g_object_set (settings, "gtk-enable-animations", FALSE, NULL);
 
-        um_carousel_purge_items (self->carousel);
+        cc_carousel_purge_items (self->carousel);
         self->other_accounts = 0;
 
         list = act_user_manager_list_users (self->um);
@@ -314,14 +312,14 @@ reload_users (CcUserPanel *self, ActUser *selected_user)
         }
         g_slist_free (list);
 
-        if (um_carousel_get_item_count (self->carousel) == 0)
+        if (cc_carousel_get_item_count (self->carousel) == 0)
                 gtk_stack_set_visible_child_name (self->stack, PAGE_NO_USERS);
         if (self->other_accounts == 0)
                 gtk_revealer_set_reveal_child (GTK_REVEALER (self->carousel), FALSE);
 
         if (selected_user)
-                item = um_carousel_find_item (self->carousel, selected_user, user_compare);
-        um_carousel_select_item (self->carousel, item);
+                item = cc_carousel_find_item (self->carousel, selected_user, user_compare);
+        cc_carousel_select_item (self->carousel, item);
 
         g_object_set (settings, "gtk-enable-animations", animations, NULL);
 }
@@ -330,12 +328,12 @@ static gint
 user_compare (gconstpointer i,
               gconstpointer u)
 {
-        UmCarouselItem *item;
+        CcCarouselItem *item;
         ActUser *user;
         gint uid_a, uid_b;
         gint result;
 
-        item = (UmCarouselItem *) i;
+        item = (CcCarouselItem *) i;
         user = ACT_USER (u);
 
         uid_a = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "uid"));
@@ -431,7 +429,7 @@ enterprise_user_revoked (GObject *source,
 {
         AsyncDeleteData *data = user_data;
         CcUserPanel *self = data->self;
-        UmRealmCommon *common = UM_REALM_COMMON (source);
+        CcRealmCommon *common = CC_REALM_COMMON (source);
         GError *error = NULL;
 
         if (g_cancellable_is_cancelled (data->cancellable)) {
@@ -439,7 +437,7 @@ enterprise_user_revoked (GObject *source,
                 return;
         }
 
-        um_realm_common_call_change_login_policy_finish (common, result, &error);
+        cc_realm_common_call_change_login_policy_finish (common, result, &error);
         if (error != NULL) {
                 show_error_dialog (self, _("Failed to revoke remotely managed user"), error);
                 g_error_free (error);
@@ -448,22 +446,22 @@ enterprise_user_revoked (GObject *source,
         async_delete_data_free (data);
 }
 
-static UmRealmCommon *
-find_matching_realm (UmRealmManager *realm_manager, const gchar *login)
+static CcRealmCommon *
+find_matching_realm (CcRealmManager *realm_manager, const gchar *login)
 {
-        UmRealmCommon *common = NULL;
+        CcRealmCommon *common = NULL;
         GList *realms, *l;
 
-        realms = um_realm_manager_get_realms (realm_manager);
+        realms = cc_realm_manager_get_realms (realm_manager);
         for (l = realms; l != NULL; l = g_list_next (l)) {
                 const gchar * const *permitted_logins;
                 gint i;
 
-                common = um_realm_object_get_common (l->data);
+                common = cc_realm_object_get_common (l->data);
                 if (common == NULL)
                         continue;
 
-                permitted_logins = um_realm_common_get_permitted_logins (common);
+                permitted_logins = cc_realm_common_get_permitted_logins (common);
                 for (i = 0; permitted_logins[i] != NULL; i++) {
                         if (g_strcmp0 (permitted_logins[i], login) == 0)
                                 break;
@@ -486,8 +484,8 @@ realm_manager_found (GObject *source,
 {
         AsyncDeleteData *data = user_data;
         CcUserPanel *self = data->self;
-        UmRealmCommon *common;
-        UmRealmManager *realm_manager;
+        CcRealmCommon *common;
+        CcRealmManager *realm_manager;
         const gchar *add[1];
         const gchar *remove[2];
         GVariant *options;
@@ -498,7 +496,7 @@ realm_manager_found (GObject *source,
                 return;
         }
 
-        realm_manager = um_realm_manager_new_finish (result, &error);
+        realm_manager = cc_realm_manager_new_finish (result, &error);
         if (error != NULL) {
                 show_error_dialog (self, _("Failed to revoke remotely managed user"), error);
                 g_error_free (error);
@@ -522,7 +520,7 @@ realm_manager_found (GObject *source,
         remove[1] = NULL;
 
         options = g_variant_new_array (G_VARIANT_TYPE ("{sv}"), NULL, 0);
-        um_realm_common_call_change_login_policy (common, "",
+        cc_realm_common_call_change_login_policy (common, "",
                                                   add, remove, options,
                                                   data->cancellable,
                                                   enterprise_user_revoked,
@@ -549,7 +547,7 @@ enterprise_user_uncached (GObject           *source,
         act_user_manager_uncache_user_finish (manager, res, &error);
         if (error == NULL) {
                 /* Find realm manager */
-                um_realm_manager_new (self->cancellable, realm_manager_found, data);
+                cc_realm_manager_new (self->cancellable, realm_manager_found, data);
         }
         else {
                 show_error_dialog (self, _("Failed to revoke remotely managed user"), error);
@@ -805,10 +803,10 @@ show_user (ActUser *user, CcUserPanel *self)
 
         self->selected_user = user;
 
-        um_user_image_set_user (self->user_icon_image, user);
-        um_user_image_set_user (self->user_icon_image2, user);
+        cc_user_image_set_user (self->user_icon_image, user);
+        cc_user_image_set_user (self->user_icon_image2, user);
 
-        um_photo_dialog_set_user (self->photo_dialog, user);
+        cc_avatar_chooser_set_user (self->avatar_chooser, user);
 
         gtk_entry_set_text (self->full_name_entry, act_user_get_real_name (user));
         gtk_widget_set_tooltip_text (GTK_WIDGET (self->full_name_entry), act_user_get_user_name (user));
@@ -1138,6 +1136,44 @@ remove_unlock_tooltip (GtkWidget *widget)
                                               G_CALLBACK (show_tooltip_now), NULL);
 }
 
+static guint
+get_num_active_admin (ActUserManager *um)
+{
+        GSList *list;
+        GSList *l;
+        guint num_admin = 0;
+
+        list = act_user_manager_list_users (um);
+        for (l = list; l != NULL; l = l->next) {
+                ActUser *u = l->data;
+                if (act_user_get_account_type (u) == ACT_USER_ACCOUNT_TYPE_ADMINISTRATOR && !act_user_get_locked (u)) {
+                        num_admin++;
+                }
+        }
+        g_slist_free (list);
+
+        return num_admin;
+}
+
+static gboolean
+would_demote_only_admin (ActUser *user)
+{
+        ActUserManager *um = act_user_manager_get_default ();
+
+        /* Prevent the user from demoting the only admin account.
+         * Returns TRUE when user is an administrator and there is only
+         * one enabled administrator. */
+
+        if (act_user_get_account_type (user) == ACT_USER_ACCOUNT_TYPE_STANDARD ||
+            act_user_get_locked (user))
+                return FALSE;
+
+        if (get_num_active_admin (um) > 1)
+                return FALSE;
+
+        return TRUE;
+}
+
 static void
 on_permission_changed (CcUserPanel *self)
 {
@@ -1358,8 +1394,8 @@ cc_user_panel_init (CcUserPanel *self)
         g_resources_register (cc_user_accounts_get_resource ());
 
         /* register types that the builder might need */
-        type = um_user_image_get_type ();
-        type = um_carousel_get_type ();
+        type = cc_user_image_get_type ();
+        type = cc_carousel_get_type ();
 
         gtk_widget_init_template (GTK_WIDGET (self));
 
@@ -1375,7 +1411,7 @@ cc_user_panel_init (CcUserPanel *self)
 
         self->login_screen_settings = settings_or_null ("org.gnome.login-screen");
 
-        self->photo_dialog = um_photo_dialog_new (GTK_WIDGET (self->user_icon_button));
+        self->avatar_chooser = cc_avatar_chooser_new (GTK_WIDGET (self->user_icon_button));
         setup_main_window (self);
 }
 
