@@ -161,14 +161,17 @@ activate_panel (CcWindow          *self,
                 CcPanelVisibility  visibility)
 {
   g_autoptr (GTimer) timer = NULL;
+  GtkWidget *sidebar_widget;
   GtkWidget *title_widget;
   gdouble ellapsed_time;
 
+  CC_ENTRY;
+
   if (!id)
-    return FALSE;
+    CC_RETURN (FALSE);
 
   if (visibility == CC_PANEL_HIDDEN)
-    return FALSE;
+    CC_RETURN (FALSE);
 
   /* clear any custom widgets */
   remove_all_custom_widgets (self);
@@ -197,6 +200,9 @@ activate_panel (CcWindow          *self,
   title_widget = cc_panel_get_title_widget (CC_PANEL (self->current_panel));
   gtk_header_bar_set_custom_title (GTK_HEADER_BAR (self->panel_headerbar), title_widget);
 
+  sidebar_widget = cc_panel_get_sidebar_widget (CC_PANEL (self->current_panel));
+  cc_panel_list_add_sidebar_widget (CC_PANEL_LIST (self->panel_list), sidebar_widget);
+
   /* Finish profiling */
   g_timer_stop (timer);
 
@@ -204,7 +210,7 @@ activate_panel (CcWindow          *self,
 
   g_debug ("Time to open panel '%s': %lfs", name, ellapsed_time);
 
-  return TRUE;
+  CC_RETURN (TRUE);
 }
 
 static void
@@ -224,11 +230,45 @@ add_current_panel_to_history (CcShell    *shell,
   g_debug ("Added '%s' to the previous panels", self->current_panel_id);
 }
 
+static gboolean
+find_iter_for_panel_id (CcWindow    *self,
+                        const gchar *panel_id,
+                        GtkTreeIter *out_iter)
+{
+  GtkTreeIter iter;
+  gboolean valid;
+
+  valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->store), &iter);
+
+  while (valid)
+    {
+      g_autofree gchar *id = NULL;
+
+      gtk_tree_model_get (GTK_TREE_MODEL (self->store),
+                          &iter,
+                          COL_ID, &id,
+                          -1);
+
+      if (g_strcmp0 (id, panel_id) == 0)
+        break;
+
+      valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (self->store), &iter);
+    }
+
+  g_assert (out_iter != NULL);
+  *out_iter = iter;
+
+  return valid;
+}
+
 static void
 update_list_title (CcWindow *self)
 {
   CcPanelListView view;
-  const gchar *title;
+  GtkTreeIter iter;
+  g_autofree gchar *title = NULL;
+
+  CC_ENTRY;
 
   view = cc_panel_list_get_view (CC_PANEL_LIST (self->panel_list));
   title = NULL;
@@ -236,15 +276,23 @@ update_list_title (CcWindow *self)
   switch (view)
     {
     case CC_PANEL_LIST_DETAILS:
-      title = _("Details");
+      title = g_strdup (_("Details"));
       break;
 
     case CC_PANEL_LIST_DEVICES:
-      title = _("Devices");
+      title = g_strdup (_("Devices"));
       break;
 
     case CC_PANEL_LIST_MAIN:
-      title = _("Settings");
+      title = g_strdup (_("Settings"));
+      break;
+
+    case CC_PANEL_LIST_WIDGET:
+      find_iter_for_panel_id (self, self->current_panel_id, &iter);
+      gtk_tree_model_get (GTK_TREE_MODEL (self->store),
+                          &iter,
+                          COL_NAME, &title,
+                          -1);
       break;
 
     case CC_PANEL_LIST_SEARCH:
@@ -254,6 +302,8 @@ update_list_title (CcWindow *self)
 
   if (title)
     gtk_header_bar_set_title (GTK_HEADER_BAR (self->header), title);
+
+  CC_EXIT;
 }
 
 static void
@@ -330,6 +380,22 @@ setup_model (CcWindow *shell)
   g_signal_connect_object (model, "row-changed", G_CALLBACK (on_row_changed_cb), shell, 0);
 }
 
+static void
+update_headerbar_buttons (CcWindow *self)
+{
+  gboolean is_main_view;
+
+  CC_ENTRY;
+
+  is_main_view = cc_panel_list_get_view (CC_PANEL_LIST (self->panel_list)) == CC_PANEL_LIST_MAIN;
+
+  gtk_widget_set_visible (self->previous_button, !is_main_view);
+  gtk_widget_set_visible (self->search_button, is_main_view);
+
+  update_list_title (self);
+
+  CC_EXIT;
+}
 
 static gboolean
 set_active_panel_from_id (CcShell      *shell,
@@ -344,8 +410,10 @@ set_active_panel_from_id (CcShell      *shell,
   GtkTreeIter iter;
   GtkWidget *old_panel;
   CcWindow *self;
-  gboolean iter_valid;
   gboolean activated;
+  gboolean found;
+
+  CC_ENTRY;
 
   self = CC_WINDOW (shell);
 
@@ -353,39 +421,24 @@ set_active_panel_from_id (CcShell      *shell,
   if (g_strcmp0 (self->current_panel_id, start_id) == 0)
     {
       g_object_set (G_OBJECT (self->current_panel), "parameters", parameters, NULL);
-      return TRUE;
+      CC_RETURN (TRUE);
     }
 
-  iter_valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->store), &iter);
-
-  /* find the details for this item */
-  while (iter_valid)
+  found = find_iter_for_panel_id (self, start_id, &iter);
+  if (!found)
     {
-      g_autofree gchar *id = NULL;
-
-      gtk_tree_model_get (GTK_TREE_MODEL (self->store), &iter,
-                          COL_NAME, &name,
-                          COL_GICON, &gicon,
-                          COL_ID, &id,
-                          COL_VISIBILITY, &visibility,
-                          -1);
-
-      if (id && strcmp (id, start_id) == 0)
-        break;
-
-      g_clear_pointer (&name, g_free);
-      g_clear_object (&gicon);
-
-      iter_valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (self->store), &iter);
+      g_warning ("Could not find settings panel \"%s\"", start_id);
+      CC_RETURN (TRUE);
     }
 
   old_panel = self->current_panel;
 
-  if (!name)
-    {
-      g_warning ("Could not find settings panel \"%s\"", start_id);
-      return TRUE;
-    }
+  gtk_tree_model_get (GTK_TREE_MODEL (self->store),
+                      &iter,
+                      COL_NAME, &name,
+                      COL_GICON, &gicon,
+                      COL_VISIBILITY, &visibility,
+                      -1);
 
   /* Activate the panel */
   activated = activate_panel (CC_WINDOW (shell), start_id, parameters, name, gicon, visibility);
@@ -395,7 +448,7 @@ set_active_panel_from_id (CcShell      *shell,
   if (!activated)
     {
       g_debug ("Failed to activate panel");
-      return TRUE;
+      CC_RETURN (TRUE);
     }
 
   if (add_to_history)
@@ -404,12 +457,16 @@ set_active_panel_from_id (CcShell      *shell,
   g_free (self->current_panel_id);
   self->current_panel_id = g_strdup (start_id);
 
+  CC_TRACE_MSG ("Current panel id: %s", start_id);
+
   if (old_panel)
     gtk_container_remove (GTK_CONTAINER (self->stack), old_panel);
 
   cc_panel_list_set_active_panel (CC_PANEL_LIST (self->panel_list), start_id);
 
-  return TRUE;
+  update_headerbar_buttons (self);
+
+  CC_RETURN (TRUE);
 }
 
 static void
@@ -463,21 +520,6 @@ show_panel_cb (CcPanelList *panel_list,
   set_active_panel_from_id (CC_SHELL (self), panel_id, NULL, TRUE, NULL);
 }
 
-static void
-panel_list_view_changed_cb (CcPanelList *panel_list,
-                            GParamSpec  *pspec,
-                            CcWindow    *self)
-{
-  gboolean is_main_view;
-
-  is_main_view = cc_panel_list_get_view (panel_list) == CC_PANEL_LIST_MAIN;
-
-  gtk_widget_set_visible (self->previous_button, !is_main_view);
-  gtk_widget_set_visible (self->search_button, is_main_view);
-
-  update_list_title (self);
-}
-
 static void
 search_entry_activate_cb (GtkEntry *entry,
                           CcWindow *self)
@@ -499,7 +541,9 @@ previous_button_clicked_cb (GtkButton *button,
   if (gtk_search_bar_get_search_mode (GTK_SEARCH_BAR (shell->search_bar)))
     gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (shell->search_bar), FALSE);
   else
-    cc_panel_list_set_view (CC_PANEL_LIST (shell->panel_list), CC_PANEL_LIST_MAIN);
+    cc_panel_list_go_previous (CC_PANEL_LIST (shell->panel_list));
+
+  update_headerbar_buttons (shell);
 }
 
 static void
@@ -769,6 +813,8 @@ cc_window_constructed (GObject *object)
   else
     cc_panel_list_activate (CC_PANEL_LIST (self->panel_list));
 
+  update_headerbar_buttons (self);
+
   G_OBJECT_CLASS (cc_window_parent_class)->constructed (object);
 }
 
@@ -843,7 +889,6 @@ cc_window_class_init (CcWindowClass *klass)
 
   gtk_widget_class_bind_template_callback (widget_class, gdk_window_set_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_development_warning_dialog_responded_cb);
-  gtk_widget_class_bind_template_callback (widget_class, panel_list_view_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, previous_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, search_entry_activate_cb);
   gtk_widget_class_bind_template_callback (widget_class, show_panel_cb);
